@@ -67,7 +67,19 @@ export type AddonWithMeta = Addon & {
 
 export type ListResult = Addon[];
 
-export type ModifyResult = (["success", Addon] | ["failure" | "error", string])[];
+export type SuccessResult = {
+  status: "success";
+  addon: Addon;
+};
+
+export type ErrorResult = {
+  status: "failure" | "error";
+  message: string;
+};
+
+export type AnyResult = SuccessResult | ErrorResult;
+
+export type MultiResult = AnyResult[];
 
 export type AddonMatch = {
   folders: { name: string; version: string }[];
@@ -79,11 +91,24 @@ export type ReconcileResult = {
   unreconciled: AddonMatch[];
 };
 
+export enum ReconciliationStage {
+  toc_ids = "toc_ids",
+  dir_names = "dir_names",
+  toc_names = "toc_names",
+}
+
 export type Version = {
   installed_version: string;
   new_version: string | null;
 };
 
+export type PydanticValidationError = {
+  ctx: object;
+  loc: string[];
+  msg: string;
+  type: string;
+};
+
 export class Api {
   getClient: () => Promise<Client>;
   profile?: string;
@@ -99,7 +124,7 @@ export class Api {
 
   async _request(requestObject: Parameters<Client["request"]>[0]) {
     const client = await this.getClient();
-    return await client.request(requestObject, null);
+    return await client.request(requestObject, 0);
   }
 
   async readProfile(profile: string): Promise<Config> {
@@ -110,8 +135,12 @@ export class Api {
     return await this._request({ method: "config/write", params: { values: config } });
   }
 
-  async enumerateProfiles(): Promise<Profiles> {
-    return await this._request({ method: "config/enumerate" });
+  async deleteProfile(profile: string): Promise<void> {
+    return await this._request({ method: "config/delete", params: { profile: profile } });
+  }
+
+  async listProfiles(): Promise<Profiles> {
+    return await this._request({ method: "config/list" });
   }
 
   async listSources(): Promise<Sources> {
@@ -133,7 +162,7 @@ export class Api {
     searchTerms: string,
     searchLimit: number,
     strategy: Strategies
-  ): Promise<ListResult> {
+  ): Promise<MultiResult> {
     return await this._request({
       method: "search",
       params: {
@@ -145,7 +174,7 @@ export class Api {
     });
   }
 
-  async resolve(defns: Defn[]): Promise<ListResult> {
+  async resolve(defns: Defn[]): Promise<MultiResult> {
     return await this._request({
       method: "resolve",
       params: { profile: this.profile, defns: defns },
@@ -155,15 +184,15 @@ export class Api {
   async modifyAddons(
     method: "install" | "update" | "remove" | "pin",
     defns: object[],
-    extraParams: object = {}
-  ): Promise<ModifyResult> {
+    extraParams: { [key: string]: any } = {}
+  ): Promise<MultiResult> {
     return await this._request({
       method: method,
       params: { profile: this.profile, defns: defns, ...extraParams },
     });
   }
 
-  async reconcile(matcher: "toc_ids" | "dir_names" | "toc_names"): Promise<ReconcileResult> {
+  async reconcile(matcher: ReconciliationStage): Promise<ReconcileResult> {
     return await this._request({
       method: "reconcile",
       params: { profile: this.profile, matcher: matcher },
@@ -178,8 +207,5 @@ export class Api {
 export const addonToDefn = (addon: Addon): Defn => ({
   source: addon.source,
   alias: addon.id,
-  strategy: {
-    type_: addon.options.strategy,
-    ...(addon.options.strategy === Strategies.version && { version: addon.version }),
-  },
+  strategy: { type_: addon.options.strategy, version: addon.version },
 });
