@@ -1,4 +1,6 @@
+import type { ChildProcessWithoutNullStreams } from "child_process";
 import { spawn } from "child_process";
+import { platform } from "process";
 import path from "path";
 import { app, BrowserWindow, dialog, ipcMain, Menu, shell } from "electron";
 import contextMenu from "electron-context-menu";
@@ -8,7 +10,13 @@ const sleep = (ms: number) => {
 };
 
 const getBinPath = () => {
-  return path.join(app.getAppPath(), "..", "..", "MacOS");
+  if (platform === "darwin") {
+    return path.join(app.getAppPath(), "..", "..", "MacOS");
+  } else if (platform === "linux") {
+    return path.join(app.getAppPath(), "..");
+  } else {
+    throw new Error(`platform "${platform}" is not supported`);
+  }
 };
 
 const spawnInstawow = () => {
@@ -29,9 +37,12 @@ const createWindow = () => {
     width: 800,
     height: 600,
     webPreferences: {
-      // contextIsolation: true,    // to investigate
-      nodeIntegration: true,
+      contextIsolation: true,
+      preload: path.join(__dirname, "preload.js"),
     },
+    // Show menu on <alt>, only applicable to Linux and Windows
+    autoHideMenuBar: true,
+    // `vibrancy`, `titleBarStyle` and `trafficLightPosition` are only applicable to macOS
     vibrancy: "titlebar",
     titleBarStyle: "hidden",
     trafficLightPosition: { x: 20, y: 40 },
@@ -74,7 +85,8 @@ let serverAddress: string;
 
 instawow.once("message", (message: any) => (serverAddress = message.address));
 
-instawow.stderr.on("data", (data) => {
+// TS does not have variadic file descriptor overloads
+(instawow as ChildProcessWithoutNullStreams).stderr.on("data", (data) => {
   if (data.toString().trim() !== "Aborted!") {
     dialog.showErrorBox("Error", `stderr: ${data}`);
   }
