@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from datetime import datetime
 import enum
 from functools import partial
 from itertools import chain
@@ -7,6 +8,7 @@ from pathlib import Path
 from textwrap import dedent, fill
 from typing import (
     TYPE_CHECKING,
+    AbstractSet,
     Any,
     Callable,
     FrozenSet,
@@ -16,7 +18,6 @@ from typing import (
     List,
     Optional as O,
     Sequence,
-    Set,
     Tuple,
     Type,
     TypeVar,
@@ -33,7 +34,7 @@ from .resolvers import Defn, Strategies
 from .utils import TocReader, cached_property, get_version, is_outdated, tabulate, uniq
 
 if TYPE_CHECKING:
-    from .manager import CliManager
+    from .manager import CliManager as CliManagerT
 
     _R = TypeVar('_R')
 
@@ -77,7 +78,7 @@ class Report:
         )
 
     def generate(self):
-        manager: CliManager = click.get_current_context().obj.m
+        manager: CliManagerT = click.get_current_context().obj.m
         if manager.config.auto_update_check:
             outdated, new_version = is_outdated()
             if outdated:
@@ -108,7 +109,7 @@ class ManagerWrapper:
         self.ctx = ctx
 
     @cached_property
-    def m(self) -> CliManager:
+    def m(self) -> CliManagerT:
         from .manager import CliManager
 
         try:
@@ -121,16 +122,16 @@ class ManagerWrapper:
         return manager
 
 
-class _PathParam(click.Path):
-    def coerce_path_result(self, value: str) -> Path:
+class PathParam(click.Path):
+    def coerce_path_result(self, value: str) -> Path:  # type: ignore
         return Path(value)
 
 
-class _EnumParam(click.Choice, Generic[_EnumT]):
+class EnumParam(click.Choice, Generic[_EnumT]):
     def __init__(
         self,
         choice_enum: Type[_EnumT],
-        excludes: Set[_EnumT] = set(),
+        excludes: AbstractSet[_EnumT] = set(),
         case_sensitive: bool = True,
     ) -> None:
         self.choice_enum = choice_enum
@@ -177,24 +178,23 @@ def _show_version(ctx: click.Context, _param: click.Parameter, value: bool):
 @click.pass_context
 def main(ctx: click.Context, log_level: bool, profile: str):
     "Add-on manager for World of Warcraft."
-    if not ctx.obj:
-        ctx.obj = ManagerWrapper(ctx)
+    ctx.obj = ManagerWrapper(ctx)
 
 
 @overload
-def parse_into_defn(manager: CliManager, value: str, *, raise_invalid: bool = True) -> Defn:
+def parse_into_defn(manager: CliManagerT, value: str, *, raise_invalid: bool = True) -> Defn:
     ...
 
 
 @overload
 def parse_into_defn(
-    manager: CliManager, value: List[str], *, raise_invalid: bool = True
+    manager: CliManagerT, value: List[str], *, raise_invalid: bool = True
 ) -> List[Defn]:
     ...
 
 
 def parse_into_defn(
-    manager: CliManager, value: Union[str, List[str]], *, raise_invalid: bool = True
+    manager: CliManagerT, value: Union[str, List[str]], *, raise_invalid: bool = True
 ) -> Union[Defn, List[Defn]]:
     if not isinstance(value, str):
         defns = (parse_into_defn(manager, v, raise_invalid=raise_invalid) for v in value)
@@ -211,32 +211,33 @@ def parse_into_defn(
 
 
 def parse_into_defn_with_strategy(
-    manager: CliManager, value: Sequence[Tuple[Strategies, str]]
+    manager: CliManagerT, value: Sequence[Tuple[Strategies, str]]
 ) -> List[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return list(map(Defn.with_strategy, defns, (s for s, _ in value)))
 
 
 def parse_into_defn_with_version(
-    manager: CliManager, value: Sequence[Tuple[str, str]]
+    manager: CliManagerT, value: Sequence[Tuple[str, str]]
 ) -> List[Defn]:
     defns = parse_into_defn(manager, [d for _, d in value])
     return list(map(Defn.with_version, defns, (v for v, _ in value)))
 
 
-def parse_into_defn_from_json_file(manager: CliManager, path: Path) -> List[Defn]:
+def parse_into_defn_from_json_file(manager: CliManagerT, path: Path) -> List[Defn]:
     faux_pkgs = models.MultiPkgModel.parse_file(path, encoding='utf-8')
     return list(map(Defn.from_pkg, cast('List[models.Pkg]', faux_pkgs.__root__)))
 
 
-def _combine_into(param_name: str, fn: Callable[[CliManager, Any], List[Any]]):
-    def combine(ctx: click.Context, _param: click.Parameter, value: Any):
-        # Add param by ``param_name`` to param dict regardless of output
-        param = ctx.params.setdefault(param_name, [])
-        if value:
-            param.extend(fn(ctx.obj.m, value))
-
-    return combine
+def combine_addons(
+    fn: Callable[[CliManagerT, Any], List[Defn]],
+    ctx: click.Context,
+    click_param: click.Parameter,
+    value: Any,
+) -> None:
+    addons = ctx.params.setdefault('addons', [])
+    if value:
+        addons.extend(fn(ctx.obj.m, value))
 
 
 excluded_strategies = {Strategies.default, Strategies.version}
@@ -244,23 +245,23 @@ excluded_strategies = {Strategies.default, Strategies.version}
 
 @main.command()
 @click.argument(
-    'addons', nargs=-1, callback=_combine_into('addons', parse_into_defn), expose_value=False
+    'addons', nargs=-1, callback=partial(combine_addons, parse_into_defn), expose_value=False
 )
 @click.option(
     '--import',
     '-i',
-    type=_PathParam(dir_okay=False, exists=True),
+    type=PathParam(dir_okay=False, exists=True),
     expose_value=False,
-    callback=_combine_into('addons', parse_into_defn_from_json_file),
+    callback=partial(combine_addons, parse_into_defn_from_json_file),
     help='Install add-ons from the output of `list -f json`.',
 )
 @click.option(
     '--with-strategy',
     '-s',
     multiple=True,
-    type=(_EnumParam(Strategies, excluded_strategies), str),
+    type=(EnumParam(Strategies, excluded_strategies), str),
     expose_value=False,
-    callback=_combine_into('addons', parse_into_defn_with_strategy),
+    callback=partial(combine_addons, parse_into_defn_with_strategy),
     metavar='<STRATEGY ADDON>...',
     help='A strategy followed by an add-on definition.  '
     'The strategies are: '
@@ -271,11 +272,11 @@ excluded_strategies = {Strategies.default, Strategies.version}
     multiple=True,
     type=(str, str),
     expose_value=False,
-    callback=_combine_into('addons', parse_into_defn_with_version),
+    callback=partial(combine_addons, parse_into_defn_with_version),
     metavar='<VERSION ADDON>...',
     help='A version followed by an add-on definition.',
 )
-@click.option('--replace', '-o', is_flag=True, default=False, help='Replace existing add-ons.')
+@click.option('--replace', is_flag=True, default=False, help='Replace unreconciled add-ons.')
 @click.pass_obj
 def install(obj: ManagerWrapper, addons: Sequence[Defn], replace: bool) -> None:
     "Install add-ons."
@@ -330,7 +331,7 @@ def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
     "Roll an add-on back to an older version."
     from .prompts import Choice, select
 
-    manager: CliManager = ctx.obj.m
+    manager: CliManagerT = ctx.obj.m
     limit = 10
 
     pkg = manager.get_pkg(addon)
@@ -338,19 +339,13 @@ def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
         Report([(addon, E.PkgNotInstalled())]).generate_and_exit()
         return  # noop
 
-    resolver = manager.resolvers[pkg.source]
-    if not resolver.supports_rollback:
+    if not manager.resolvers[pkg.source].supports_rollback:
         Report(
             [(addon, E.PkgFileUnavailable('source does not support rollback'))]
         ).generate_and_exit()
 
     if undo:
-        Report(
-            chain(
-                manager.run(manager.remove([addon])).items(),
-                manager.run(manager.install([addon], replace=False)).items(),
-            )
-        ).generate_and_exit()
+        Report(manager.run(manager.update([addon], True)).items()).generate_and_exit()
 
     versions = (
         manager.database.query(models.PkgVersionLog)
@@ -376,12 +371,7 @@ def rollback(ctx: click.Context, addon: Defn, undo: bool) -> None:
         f'Select version of {reconstructed_defn} for rollback', choices
     ).unsafe_ask()
     Report(
-        chain(
-            manager.run(manager.remove([reconstructed_defn])).items(),
-            manager.run(
-                manager.install([reconstructed_defn.with_version(selection)], replace=False)
-            ).items(),
-        )
+        manager.run(manager.update([reconstructed_defn.with_version(selection)], True)).items()
     ).generate_and_exit()
 
 
@@ -418,7 +408,7 @@ def reconcile(ctx: click.Context, auto: bool, list_unreconciled: bool) -> None:
         '''
     )
 
-    manager: CliManager = ctx.obj.m
+    manager: CliManagerT = ctx.obj.m
 
     def prompt_one(addons: List[AddonFolder], pkgs: List[models.Pkg]) -> Union[Defn, Tuple[()]]:
         def construct_choice(pkg: models.Pkg):
@@ -501,7 +491,7 @@ def search(ctx: click.Context, search_terms: str, limit: int) -> None:
     "Search for add-ons to install."
     from .prompts import PkgChoice, checkbox, confirm
 
-    manager: CliManager = ctx.obj.m
+    manager: CliManagerT = ctx.obj.m
 
     pkgs = manager.run(manager.search(search_terms, limit))
     if pkgs:
@@ -525,7 +515,7 @@ class ListFormats(enum.Enum):
     '--format',
     '-f',
     'output_format',
-    type=_EnumParam(ListFormats),
+    type=EnumParam(ListFormats),
     default=ListFormats.simple.name,
     show_default=True,
     help='Change the output format.',
@@ -694,24 +684,30 @@ def list_installed_wago_auras(obj: ManagerWrapper, account: str) -> None:
     config = BuilderConfig(account=account)
     aura_groups = WaCompanionBuilder(obj.m, config).extract_installed_auras()
     installed_auras = sorted(
-        (g.Meta.model.__name__, fill(a.id, width=30, max_lines=1), a.url)
+        (g.filename, fill(a.id, width=30, max_lines=1), a.url)
         for g in aura_groups
         for v in g.entries.values()
         for a in v
         if not a.parent
     )
-    click.echo(tabulate([('type', 'name', 'URL'), *installed_auras]))
+    click.echo(tabulate([('in file', 'name', 'URL'), *installed_auras]))
 
 
 @main.command(hidden=True)
-@click.argument('filename', type=_PathParam(dir_okay=False))
-def generate_catalogue(filename: Path) -> None:
+@click.argument('filename', type=PathParam(dir_okay=False))
+@click.option(
+    '--age-cutoff',
+    type=str,
+    default=None,
+    callback=lambda _, __, v: v and datetime.fromisoformat(v),
+)
+def generate_catalogue(filename: Path, age_cutoff: O[datetime]) -> None:
     "Generate the master catalogue."
     import asyncio
 
     from .resolvers import MasterCatalogue
 
-    catalogue = asyncio.run(MasterCatalogue.collate())
+    catalogue = asyncio.run(MasterCatalogue.collate(age_cutoff))
     filename.write_text(catalogue.json(indent=2), encoding='utf-8')
     filename.with_suffix(f'.compact{filename.suffix}').write_text(
         catalogue.json(separators=(',', ':')), encoding='utf-8'
@@ -724,10 +720,10 @@ def listen(ctx: click.Context) -> None:
     "Fire up the WebSocket server."
     import asyncio
 
-    from .json_rpc_server import listen
+    from . import json_rpc_server
 
     dummy_config = Config.get_dummy_config(profile='__jsonrpc__').ensure_dirs()
     log_level = ctx.find_root().params['log_level']
     setup_logging(dummy_config, log_level)
     _override_loop_policy()
-    asyncio.run(listen())
+    asyncio.run(json_rpc_server.listen())
