@@ -46,6 +46,7 @@ from .resolvers import (
     Strategies,
     TukuiResolver,
     WowiResolver,
+    normalise_names,
 )
 from .utils import (
     bucketise,
@@ -250,12 +251,12 @@ def _error_out(error: BaseException) -> Callable[[], Awaitable[NoReturn]]:
     return inner
 
 
-async def _capture_exc(coro: Callable[..., Awaitable[Any]]) -> E.ManagerResult:
+async def _capture_exc_async(coro: Callable[..., Awaitable[Any]]) -> Any:
     from aiohttp import ClientError
 
     try:
         return await coro()
-    except E.ManagerError as error:
+    except (E.ManagerError, E.InternalError) as error:
         return error
     except ClientError as error:
         logger.opt(exception=True).debug('network error')
@@ -361,7 +362,7 @@ class Manager:
                 self.database.query(Pkg)
                 .filter(
                     Pkg.source == defn.source,
-                    (Pkg.id == defn.alias) | (Pkg.slug == defn.alias) | (Pkg.id == defn.source_id),
+                    (Pkg.id == defn.alias) | (Pkg.slug == defn.alias) | (Pkg.id == defn.id),
                 )
                 .first()
             )
@@ -407,11 +408,11 @@ class Manager:
                     missing_ok=True,
                 )
             else:
-                foreign_conflicts = top_level_folders & {
+                unreconciled_conflicts = top_level_folders & {
                     f.name for f in self.config.addon_dir.iterdir()
                 }
-                if foreign_conflicts:
-                    raise E.PkgConflictsWithForeign(foreign_conflicts)
+                if unreconciled_conflicts:
+                    raise E.PkgConflictsWithUnreconciled(unreconciled_conflicts)
 
             extract(self.config.addon_dir)
             pkg.folders = [PkgFolder(name=f) for f in sorted(top_level_folders)]
@@ -437,11 +438,11 @@ class Manager:
             if installed_conflicts:
                 raise E.PkgConflictsWithInstalled(installed_conflicts)
 
-            foreign_conflicts = top_level_folders - {f.name for f in old_pkg.folders} & {
+            unreconciled_conflicts = top_level_folders - {f.name for f in old_pkg.folders} & {
                 f.name for f in self.config.addon_dir.iterdir()
             }
-            if foreign_conflicts:
-                raise E.PkgConflictsWithForeign(foreign_conflicts)
+            if unreconciled_conflicts:
+                raise E.PkgConflictsWithUnreconciled(unreconciled_conflicts)
 
             trash(
                 [self.config.addon_dir / f.name for f in old_pkg.folders],
@@ -479,8 +480,8 @@ class Manager:
             label = 'Synchronising master catalogue'
             url = (
                 'https://raw.githubusercontent.com/layday/instawow-data/data/'
-                'master-catalogue-v1.compact.json'
-            )  # v1
+                'master-catalogue-v2.compact.json'
+            )  # v2
             raw_catalogue = await cache_json_response(self, url, 4, 'hours', label=label)
             self.catalogue = MasterCatalogue.parse_obj(raw_catalogue)
 
@@ -515,7 +516,13 @@ class Manager:
         await self.synchronise()
 
         defns_by_source = bucketise(defns, key=lambda v: v.source)
-        results = await gather(self.resolvers[s].resolve(b) for s, b in defns_by_source.items())
+        results: List[Union[Dict[Defn, Any], E.ManagerError]] = await gather(
+            (
+                _capture_exc_async(partial(self.resolvers[s].resolve, b))
+                for s, b in defns_by_source.items()
+            ),
+            False,
+        )
         results_by_defn = chain_dict(
             defns,
             None,
@@ -533,38 +540,34 @@ class Manager:
     ) -> Dict[Defn, Pkg]:
         "Search the master catalogue for packages by name."
         import heapq
-        import string
 
         from jellyfish import jaro_winkler_similarity
 
-        trans_table = str.maketrans(dict.fromkeys(string.punctuation, ' '))
-
-        def normalise(value: str):
-            return value.casefold().translate(trans_table).strip()
-
         await self.synchronise()
 
+        w = 0.5  # Weighing edit distance and download score equally
+        normalise = normalise_names()
+
         s = normalise(search_terms)
-        tokens_to_defns = bucketise(
+        defns_by_token = bucketise(
             (
-                (normalise(i.name), i)
+                (i.normalised_name, i)
                 for i in self.catalogue.__root__
-                if self.config.game_flavour in i.compatibility
+                if self.config.game_flavour in i.game_compatibility
             ),
             key=lambda v: v[0],
         )
-
-        # TODO: weigh matches under threshold against download count
         matches = heapq.nlargest(
             limit,
-            ((jaro_winkler_similarity(s, n), n) for n in tokens_to_defns),
+            ((jaro_winkler_similarity(s, n), n) for n in defns_by_token),
             key=lambda v: v[0],
         )
-        defns = [
-            Defn.get(i.source, i.id).with_(strategy={'type_': strategy})
-            for _, m in matches
-            for _, i in tokens_to_defns[m]
-        ]
+        weighted_entries = sorted(
+            (-(s * w + i.derived_download_score * (1 - w)), i)
+            for s, m in matches
+            for _, i in defns_by_token[m]
+        )
+        defns = [Defn.get(i.source, i.id).with_strategy(strategy) for _, i in weighted_entries]
         resolve_results = await self.resolve(defns)
         pkgs_by_defn = {d.with_(alias=r.slug): r for d, r in resolve_results.items() if is_pkg(r)}
         return pkgs_by_defn
@@ -602,7 +605,7 @@ class Manager:
                 for (d, p), a in zip(installables, archives)
             ),
         )
-        results = {d: await _capture_exc(c) for d, c in result_coros.items()}
+        results = {d: await _capture_exc_async(c) for d, c in result_coros.items()}
         return results
 
     @_with_lock('change state')
@@ -620,7 +623,7 @@ class Manager:
         # the benefit of resolving installed-but-renamed packages -
         # the slug is transient but the ID is not
         defns_to_pkgs = {
-            (Defn.from_pkg(p) if not retain_strategy and p else d.with_(source_id=p and p.id)): p
+            (Defn.from_pkg(p) if not retain_strategy and p else d.with_(id=p and p.id)): p
             for d, p in zip(defns, (self.get_pkg(d) for d in defns))
         }
         resolve_results = await self.resolve([d for d, p in defns_to_pkgs.items() if p])
@@ -648,7 +651,7 @@ class Manager:
                 for (d, *p), a in zip(updatables, archives)
             ),
         )
-        results = {d: await _capture_exc(c) for d, c in result_coros.items()}
+        results = {d: await _capture_exc_async(c) for d, c in result_coros.items()}
         return results
 
     @_with_lock('change state')
@@ -660,7 +663,7 @@ class Manager:
             _error_out(E.PkgNotInstalled()),
             ((d, partial(t(self.remove_pkg), p)) for d, p in zip(defns, maybe_pkgs) if p),
         )
-        results = {d: await _capture_exc(c) for d, c in result_coros.items()}
+        results = {d: await _capture_exc_async(c) for d, c in result_coros.items()}
         return results
 
     @_with_lock('change state')
