@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from datetime import datetime
+from datetime import datetime, timezone
 import enum
 from itertools import chain, count, takewhile
 import re
@@ -23,12 +23,12 @@ from typing_extensions import Literal, TypedDict, get_args
 from yarl import URL
 
 from . import exceptions as E, models as m
-from .utils import cached_property, gather, run_in_thread as t, slugify, uniq
+from .utils import bucketise, cached_property, gather, run_in_thread as t, slugify, uniq
 
 if TYPE_CHECKING:
-    from pydantic.fields import ModelField
+    from pydantic.fields import ModelField as ModeFieldT
 
-    from .manager import Manager
+    from .manager import Manager as ManagerT
 
 
 class Strategies(enum.Enum):
@@ -41,6 +41,9 @@ class Strategies(enum.Enum):
 
 
 class HashableModel(BaseModel):
+    class Config:
+        allow_mutation = False
+
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, self.__class__):
             return NotImplemented
@@ -52,7 +55,7 @@ class HashableModel(BaseModel):
 
 class BaseStrategy(HashableModel):
     @validator('type_', check_fields=False, pre=True)
-    def _parse_strategy(cls, value: Any, field: ModelField) -> Strategies:
+    def _parse_strategy(cls, value: Any, field: ModeFieldT) -> Strategies:
         accepted_strategies = get_args(field.type_)
         strategy = Strategies.__members__.get(value) or value
         if strategy in accepted_strategies:
@@ -77,7 +80,7 @@ class VersionStrategy(BaseStrategy):
 
 class Defn(HashableModel):
     source: str
-    source_id: O[str] = None
+    id: O[str] = None
     alias: str
     strategy: Union[SimpleStrategy, VersionStrategy] = SimpleStrategy(type_=Strategies.default)
 
@@ -88,7 +91,7 @@ class Defn(HashableModel):
         )
         return cls(
             source=pkg.source,
-            source_id=pkg.id,
+            id=pkg.id,
             alias=pkg.slug,
             strategy={'type_': pkg.options.strategy, **strategy_kwargs},
         )
@@ -110,13 +113,39 @@ class Defn(HashableModel):
         return f'{self.source}:{self.alias}'
 
 
+def normalise_names():
+    import string
+
+    trans_table = str.maketrans(dict.fromkeys(string.punctuation, ' '))
+
+    def normalise(value: str):
+        return ' '.join(value.casefold().translate(trans_table).split())
+
+    return normalise
+
+
+class _CatalogueEntryDefaultFields(TypedDict):
+    source: str
+    id: str
+    slug: str
+    name: str
+    game_compatibility: Set[Literal['retail', 'classic']]
+    folders: Sequence[Sequence[str]]
+    download_count: int
+    last_updated: Any
+
+
 class _CatalogueEntry(BaseModel):
     source: str
     id: str
-    slug: str = ''
+    slug: str
     name: str
-    compatibility: Set[Literal['retail', 'classic']] = {'retail', 'classic'}
-    folders: List[List[str]] = []
+    game_compatibility: Set[Literal['retail', 'classic']]
+    folders: List[List[str]]
+    download_count: int
+    last_updated: datetime
+    normalised_name: str
+    derived_download_score: float
 
 
 class MasterCatalogue(BaseModel):
@@ -126,7 +155,7 @@ class MasterCatalogue(BaseModel):
         keep_untouched = (cast(Any, cached_property),)
 
     @classmethod
-    async def collate(cls) -> MasterCatalogue:
+    async def collate(cls, age_cutoff: O[datetime]) -> MasterCatalogue:
         from types import SimpleNamespace
 
         from .manager import init_web_client
@@ -134,9 +163,26 @@ class MasterCatalogue(BaseModel):
         resolvers = (CurseResolver, WowiResolver, TukuiResolver)
 
         async with init_web_client() as web_client:
-            faux_manager = cast('Manager', SimpleNamespace(web_client=web_client))
+            faux_manager = cast('ManagerT', SimpleNamespace(web_client=web_client))
             items = [a for r in resolvers async for a in r(faux_manager).collect_items()]
-        catalogue = cls(__root__=items)
+
+        normalise = normalise_names()
+        most_downloads_per_source = {
+            s: max(e['download_count'] for e in i)
+            for s, i in bucketise(items, key=lambda v: v['source']).items()
+        }
+        entries = (
+            _CatalogueEntry(
+                **i,
+                normalised_name=normalise(i['name']),
+                derived_download_score=i['download_count']
+                / most_downloads_per_source[i['source']],
+            )
+            for i in items
+        )
+        if age_cutoff:
+            entries = (e for e in entries if e.last_updated >= age_cutoff)
+        catalogue = cls.parse_obj(list(entries))
         return catalogue
 
     @cached_property
@@ -149,7 +195,7 @@ class Resolver:
     name: ClassVar[str]
     strategies: ClassVar[Set[Strategies]]
 
-    def __init__(self, manager: Manager) -> None:
+    def __init__(self, manager: ManagerT) -> None:
         self.manager = manager
 
     def __init_subclass__(cls) -> None:
@@ -179,7 +225,7 @@ class Resolver:
         "Resolve an individual definition into a package."
         raise NotImplementedError
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
         "Yield add-ons from source for cataloguing."
         return
         yield
@@ -225,8 +271,10 @@ if TYPE_CHECKING:
         name: str  # User-facing add-on name
         websiteUrl: str  # e.g. 'https://www.curseforge.com/wow/addons/molinari'
         summary: str  # One-line description of the add-on
+        downloadCount: int  # Total number of downloads
         latestFiles: List[CurseAddon_File]
         slug: str  # URL slug; 'molinari' in 'https://www.curseforge.com/wow/addons/molinari'
+        dateReleased: str  # ISO datetime of latest release
 
 
 class CurseResolver(Resolver):
@@ -262,8 +310,7 @@ class CurseResolver(Resolver):
         from .manager import cache_json_response
 
         defns_to_ids = {
-            d: (d.source_id or self.manager.catalogue.curse_slugs.get(d.alias) or d.alias)
-            for d in defns
+            d: (d.id or self.manager.catalogue.curse_slugs.get(d.alias) or d.alias) for d in defns
         }
         numeric_ids = {i for i in defns_to_ids.values() if i.isdigit()}
         try:
@@ -383,7 +430,7 @@ class CurseResolver(Resolver):
             deps=[m.PkgDep(id=d['addonId']) for d in file['dependencies'] if d['type'] == 3],
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
         classic_version_prefix = '1.13'
         flavours = ('retail', 'classic')
 
@@ -415,13 +462,15 @@ class CurseResolver(Resolver):
                     for f in item['latestFiles']
                     if not f['exposeAsAlternative']
                 )
-                yield _CatalogueEntry(
+                yield _CatalogueEntryDefaultFields(
                     source=self.source,
-                    id=item['id'],
+                    id=str(item['id']),
                     slug=item['slug'],
                     name=item['name'],
-                    compatibility=set(excise_compatibility(item['latestFiles'])),
+                    game_compatibility=set(excise_compatibility(item['latestFiles'])),
                     folders=folders,
+                    download_count=item['downloadCount'],
+                    last_updated=item['dateReleased'],
                 )
 
 
@@ -435,7 +484,7 @@ if TYPE_CHECKING:
         UIName: str  # User-facing add-on name
         UIAuthorName: str
 
-    class WowiCompatibilityEntry:
+    class WowiCompatibilityEntry(TypedDict):
         version: str  # Game version, e.g. '8.3.0'
         name: str  # Xpac or patch name, e.g. "Visions of N'Zoth" for 8.3.0
 
@@ -562,16 +611,20 @@ class WowiResolver(Resolver):
             options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
         async with self.manager.web_client.get(self.list_api_url) as response:
             list_api_items: List[WowiListApiItem] = await response.json()
 
         for list_item in list_api_items:
-            yield _CatalogueEntry(
+            yield _CatalogueEntryDefaultFields(
                 source=self.source,
                 id=list_item['UID'],
                 name=list_item['UIName'],
+                slug='',
                 folders=[list_item['UIDir']],
+                game_compatibility={'classic', 'retail'},
+                download_count=int(list_item['UIDownloadTotal']),
+                last_updated=list_item['UIDate'],
             )
 
 
@@ -601,7 +654,7 @@ if TYPE_CHECKING:
         category: str
         changelog: str
         donate_url: str
-        downloads: str
+        downloads: str  # Not a mistake, it is actually a string
         id: str
         last_download: str
         # ISO *datetime* with space sep and without an offset, e.g. '2020-02-02 12:12:20'
@@ -666,37 +719,38 @@ class TukuiResolver(Resolver):
             description=addon['small_desc'],
             url=addon['web_url'],
             download_url=addon['url'],
-            date_published=datetime.fromisoformat(addon['lastupdate']),
+            date_published=datetime.fromisoformat(addon['lastupdate']).astimezone(timezone.utc),
             version=addon['version'],
             options=m.PkgOptions(strategy=defn.strategy.type_.name),
         )
 
-    async def collect_items(self) -> AsyncIterable[_CatalogueEntry]:
-        for query, param, compatibility in [
+    async def collect_items(self) -> AsyncIterable[_CatalogueEntryDefaultFields]:
+        for query, param, game_compatibility in [
             ('ui', 'tukui', 'retail'),
             ('ui', 'elvui', 'retail'),
             ('addons', 'all', 'retail'),
             ('classic-addons', 'all', 'classic'),
         ]:
-            url = self.api_url.with_query({query: param})
-            async with self.manager.web_client.get(url) as response:
+            async with self.manager.web_client.get(
+                self.api_url.with_query({query: param})
+            ) as response:
                 metadata = await response.json(content_type=None)  # text/html
 
-            if query == 'ui':
-                yield _CatalogueEntry(
+            for item in cast(
+                'List[Union[TukuiUi, TukuiAddon]]', [metadata] if query == 'ui' else metadata
+            ):
+                yield _CatalogueEntryDefaultFields(
                     source=self.source,
-                    id=metadata['id'],
-                    name=metadata['name'],
-                    compatibility={compatibility},
+                    id=str(item['id']),
+                    slug='',
+                    name=item['name'],
+                    folders=[],
+                    game_compatibility={game_compatibility},
+                    download_count=int(item['downloads']),
+                    last_updated=datetime.fromisoformat(item['lastupdate']).astimezone(
+                        timezone.utc
+                    ),
                 )
-            else:
-                for item in metadata:
-                    yield _CatalogueEntry(
-                        source=self.source,
-                        id=item['id'],
-                        name=item['name'],
-                        compatibility={compatibility},
-                    )
 
 
 if TYPE_CHECKING:
