@@ -1,4 +1,5 @@
 from functools import lru_cache, partial
+import os
 from pathlib import Path
 import re
 
@@ -46,7 +47,8 @@ def make_zip(name):
 
 @pytest.fixture(scope='session')
 def temp_dir(tmp_path_factory):
-    yield tmp_path_factory.mktemp('temp')
+    os.environ['INSTAWOW_TEMP_DIR'] = str(tmp_path_factory.mktemp('temp'))
+    yield os.environ['INSTAWOW_TEMP_DIR']
 
 
 @pytest.fixture(params=['retail', 'classic'])
@@ -63,9 +65,7 @@ def full_config(tmp_path, partial_config):
 
 @pytest.fixture
 def manager(full_config):
-    config = Config(**full_config).write()
-    manager = Manager.from_config(config)
-    yield manager
+    yield Manager.from_config(Config(**full_config).write())
 
 
 @pytest.fixture
@@ -204,7 +204,7 @@ def mock_tukui(aresponses, JsonResponse, mock_master_catalogue):
 
 @pytest.fixture
 @should_mock
-def mock_github(aresponses, JsonResponse):
+def mock_github(aresponses, JsonResponse, mock_master_catalogue):
     aresponses.add(
         'api.github.com',
         '/repos/AdiAddons/AdiButtonAuras',
@@ -280,6 +280,7 @@ def mock_github(aresponses, JsonResponse):
 @pytest.fixture
 def mock_all(
     mock_pypi,
+    mock_master_catalogue,
     mock_curse,
     mock_wowi,
     mock_tukui,
